using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;
using System.Drawing.Printing;

namespace Utils4226
{
    public static class Printing
    {

        public static RectangleF GetRealMarginBounds(
            PrintPageEventArgs e, bool bPreview)
        {
            RectangleF marginBounds = e.MarginBounds;

            if (!bPreview)
            {
                marginBounds.Offset((int)(-e.PageSettings.HardMarginX),
                                    (int)(-e.PageSettings.HardMarginY));
            }

            return ToPageFromPrinter(e.Graphics, marginBounds);

        }

        public static RectangleF GetRealPageBounds(
            PrintPageEventArgs e, bool preview)
        {
            //If in preview mode, then do not need to worry about edges of paper.
            //Translate from 1/100 to page units.
            if (preview) return ToPageFromPrinter(e.Graphics, e.PageBounds);

            //Translate from world to page, in case there is an active transform.
            //This will give the transformed vcb. Otherwise, the clip bounds are
            //always to the edge of the paper, in the coordinates of the graphics
            //context.
            PointF[] points = new PointF[] { 
                new PointF(
                    e.Graphics.VisibleClipBounds.X,
                    e.Graphics.VisibleClipBounds.Y),
                new PointF(
                    e.Graphics.VisibleClipBounds.Right,
                    e.Graphics.VisibleClipBounds.Bottom)
            };
            e.Graphics.TransformPoints(CoordinateSpace.Page,
                CoordinateSpace.World, points);
            return new RectangleF(points[0], new SizeF(points[1]));
        }

        public static RectangleF ToPageFromPrinter(Graphics g, RectangleF rectf)
        {
            PointF ptLoc = rectf.Location;
            PointF ptDim = new PointF(rectf.Size.Width, rectf.Size.Height);
            PointF[] ptfArray = new PointF[]{
                ToDevice(g, ptLoc), 
                ToDevice(g, ptDim)};
            g.TransformPoints(CoordinateSpace.Page, CoordinateSpace.Device,
                ptfArray);
            return new RectangleF(
                ptfArray[0],
                new SizeF(ptfArray[1]));
        }

        public static Point ToDevice(Graphics g, PointF ptPrintUnit)
        {
            return new Point(
                (int)(ptPrintUnit.X * g.DpiX / 100),
                (int)(ptPrintUnit.Y * g.DpiY / 100));
        }

    }
}

