﻿#region Using directives

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Text;
using System.Windows.Forms;

#endregion

namespace BrushesSample {
  partial class BrushesForm : Form {

      BufferedGraphicsContext bufferContext;
    public BrushesForm() {
        InitializeComponent();
        bufferContext = new BufferedGraphicsContext();
        bufferContext.MaximumBuffer = this.ClientRectangle.Size;
    }

    
    private void BrushesForm_Paint(object sender, PaintEventArgs e) {
      Graphics g = e.Graphics;
      int x = 0;
      int y = 0;
      Brush whiteBrush = System.Drawing.Brushes.White;
      Brush blackBrush = System.Drawing.Brushes.Black;

      // Draw SolidBrush
      using( Brush brush = new SolidBrush(Color.DarkBlue) ) {
        g.FillRectangle(brush, x, y, width, height);
        g.DrawString(brush.ToString(), this.Font, whiteBrush, x, y);
        y += height;
      }

      // Draw TextureBrush
      using( Brush brush = new TextureBrush(Properties.Resources.Santa_Fe_Stucco) ) {
        g.FillRectangle(brush, x, y, width, height);
        g.DrawString(brush.ToString(), this.Font, whiteBrush, x, y);
        y += height;
      }

      // Draw HatchBrush
      using( Brush brush = new HatchBrush(HatchStyle.Divot, Color.DarkBlue, Color.White) ) {
        g.FillRectangle(brush, x, y, width, height);
        g.DrawString(brush.ToString(), this.Font, blackBrush, x, y);
        y += height;
      }

      // Draw LinearGradientBrush
      using( Brush brush = new LinearGradientBrush(
          new Rectangle(x, y, width, height), Color.DarkBlue, Color.White, fAngle) ) {
        g.FillRectangle(brush, x, y, width, height);
        g.DrawString(brush.ToString(), this.Font, blackBrush, x, y);
        y += height;
      }

      // Draw PathGradientBrush
      Point[] points = new Point[] { new Point(x, y),
                                     new Point(x + width, y),
                                     new Point(x + width, y + height),
                                     new Point(x, y + height) };
      using (PathGradientBrush brush = new PathGradientBrush(points))
      {
          brush.CenterColor = Color.Red;
          //Console.WriteLine("ptCenter: {0}", ptCenter);
          //if (ptCenter != Point.Empty)
          //{
              brush.CenterPoint = ptCenter;
          //}
          //brush.SurroundColors = new Color[4] { Color.Green, Color.Blue, Color.Yellow, Color.Purple };
          g.FillRectangle(brush, x, y, width, height);
        g.DrawString(brush.ToString(), this.Font, blackBrush, x, y);
        y += height;
      }
    }

      private void toolStripComboBoxLinearAngle_Click(object sender, EventArgs e)
      {

      }

      float fAngle = 45.0f;
      private void toolStripComboBoxLinearAngle_SelectedIndexChanged(object sender, EventArgs e)
      {
          fAngle = (float)Convert.ToDouble(toolStripComboBoxLinearAngle.Text);
          Invalidate();
      }

      private void centerToolStripMenuItem_Click(object sender, EventArgs e)
      {

      }

      private void toolStripComboBoxCenter_Click(object sender, EventArgs e)
      {

      }
      
      int width;
      int height;
      Rectangle rectGradient;
      private void BrushesForm_Load(object sender, EventArgs e)
      {
          width = this.ClientRectangle.Width;
          height = this.ClientRectangle.Height / 5;
          rectGradient = new Rectangle(0, height * 4, width, height);
          bufferContext.MaximumBuffer = rectGradient.Size;
      }

      private void BrushesForm_Resize(object sender, EventArgs e)
      {
          width = this.ClientRectangle.Width;
          height = this.ClientRectangle.Height / 5;
          rectGradient = new Rectangle(0, height * 4, width, height);
          bufferContext.MaximumBuffer = rectGradient.Size;
      }

      bool bDown = false;
      private void BrushesForm_MouseDown(object sender, MouseEventArgs e)
      {
          bDown = true;
      }

      Point ptCenter = Point.Empty;
      private void BrushesForm_MouseMove(object sender, MouseEventArgs e)
      {
          if (bDown)
          {
              ptCenter = e.Location;
              if (checkBoxManual.Checked)
              {
                  BufferedPaint();
              } 
              else if (checkBoxInvalidateRect.Checked)
              {
                  Invalidate(new Rectangle(0, height * 4, width, height));
              }
              else
              {
                  Invalidate(true);
              }
          }
      }

      private void BrushesForm_MouseUp(object sender, MouseEventArgs e)
      {
          if (bDown)
          {
              ptCenter = e.Location;
              bDown = false;
              Invalidate(true);
          }
      }

      private void checkBoxBuffered_CheckedChanged(object sender, EventArgs e)
      {
          this.SetStyle(ControlStyles.OptimizedDoubleBuffer, checkBoxBuffered.Checked);
          this.SetStyle(ControlStyles.AllPaintingInWmPaint, checkBoxBuffered.Checked);
          checkBoxInvalidateRect.Visible = !checkBoxBuffered.Checked;
          checkBoxManual.Visible = !checkBoxBuffered.Checked;
      }

      private void checkBoxInvalidateRect_CheckedChanged(object sender, EventArgs e)
      {

      }

      private void BufferedPaint()
      {
          int x = 0;
          int y = height * 4;
          Point[] points = new Point[] { new Point(x, y),
                                     new Point(x + width, y),
                                     new Point(x + width, y + height),
                                     new Point(x, y + height) };
          using (Graphics g = this.CreateGraphics())
          {
              using (BufferedGraphics frame =
                  bufferContext.Allocate(g, rectGradient))
              {
                  using (PathGradientBrush brush = new PathGradientBrush(points))
                  {
                      brush.CenterColor = Color.Red;
                      //Console.WriteLine("ptCenter: {0}", ptCenter);
                      //if (ptCenter != Point.Empty)
                      //{
                      brush.CenterPoint = ptCenter;
                      //}
                      //brush.SurroundColors = new Color[4] { Color.Green, Color.Blue, Color.Yellow, Color.Purple };
                      frame.Graphics.FillRectangle(brush, x, y, width, height);
                      frame.Graphics.DrawString(brush.ToString(), this.Font, Brushes.Black, x, y);
                      frame.Render();
                      y += height;
                  }
              }
          }
      }

      private void checkBoxManual_CheckedChanged(object sender, EventArgs e)
      {
          checkBoxBuffered.Visible = !checkBoxManual.Checked;
          checkBoxInvalidateRect.Visible = !checkBoxManual.Checked;
      }
  }
}