﻿#region Using directives

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Text;
using System.Windows.Forms;

#endregion

namespace BrushesSample
{
    partial class PathGradientBrushesForm : Form
    {
        public PathGradientBrushesForm()
        {
            InitializeComponent();
        }

        private void PathGradientBrushesForm_Paint(object sender, PaintEventArgs e)
        {
            Graphics g = e.Graphics;
            int width = this.ClientRectangle.Width / 2;
            int height = this.ClientRectangle.Height / 2;

            Point[] triPoints = new Point[] { new Point(width/4, 0),
                                        new Point(0, height/2),
                                        new Point(width/2, height/2), };
            using (PathGradientBrush brush = new PathGradientBrush(triPoints))
            {
                brush.WrapMode = WrapMode.TileFlipY;
                int x = 0;
                int y = 0;
                Blend blend = new Blend();
                blend.Factors = new float[] { 0.0f, .25f, .5f, .25f, 0.0f };
                blend.Positions = new float[] { 0.0f, .25f, .5f, .75f, 1.0f };
                brush.Blend = blend;
                g.FillRectangle(brush, x, y, width, height);
            }

            //Point[] quadPoints = new Point[] { new Point(0, 0), 
            //                             new Point(width, 0), 
            //                             new Point(width, height), 
            //                             new Point(0, height) };
            //using (PathGradientBrush brush = new PathGradientBrush(quadPoints))
            using (GraphicsPath rect = new GraphicsPath())
            {
                rect.AddRectangle(new Rectangle(0, 0, width, height));
                using (PathGradientBrush brush = new PathGradientBrush(rect))
                {
                    brush.CenterColor = Color.Black;
                    // defaults to Clamp, which won't draw anywhere except upper left of client
                    brush.WrapMode = WrapMode.Tile;
                    int x = width;
                    int y = 0;
                    g.FillRectangle(brush, x, y, width, height);
                }
            }

            Point[] diamondPoints = new Point[] { new Point(width / 2, 0), new Point(width, height / 2), new Point(width / 2, height), new Point(0, height / 2), };
            using (PathGradientBrush brush =
                     new PathGradientBrush(diamondPoints))
            {
                brush.WrapMode = WrapMode.Tile;
                brush.SurroundColors = new Color[] {
                    Color.LightBlue, Color.Pink, Color.LightGreen, Color.Red };
                brush.CenterPoint = new Point(width/4, height / 2);
                int x = 0;
                int y = height;
                g.FillRectangle(brush, x, y, width, height);
            }

            using (GraphicsPath circle = new GraphicsPath())
            {
                circle.AddEllipse(0, 0, width, height);
                using (PathGradientBrush brush = new PathGradientBrush(circle))
                {
                    brush.WrapMode = WrapMode.Tile;                    
                    brush.SurroundColors = new Color[] { Color.Purple, Color.Yellow };
                    ColorBlend blend = new ColorBlend();
                    blend.Colors = new Color[] { Color.Red, Color.Yellow, Color.Green };
                    blend.Positions = new float[] { 0.0f, 0.5f, 1.0f };
                    brush.InterpolationColors = blend;
                    brush.CenterColor = Color.Black;
                    if (ptCenter != Point.Empty) brush.CenterPoint = ptCenter;
                    ptCenter = Point.Round(brush.CenterPoint);
                    int x = width;
                    int y = height;
                    g.FillRectangle(brush, x, y, width, height);
                }
            }
        }

        bool bHitCenter = false;
        Point ptDown = Point.Empty;
        Point ptCenter = Point.Empty;
        private void PathGradientBrushesForm_MouseDown(object sender, MouseEventArgs e)
        {
            Point pt = new Point(
                e.X - this.ClientRectangle.Width / 2,
                e.Y - this.ClientRectangle.Height / 2
            );
            if (IsNear(ptCenter, pt, 50))
            {
                bHitCenter = true;
                ptDown = e.Location;
            }
        }

        private void PathGradientBrushesForm_MouseMove(object sender, MouseEventArgs e)
        {
            if (bHitCenter)
            {
                ptCenter = new Point(
                    ptCenter.X + e.X - ptDown.X,
                    ptCenter.Y + e.Y - ptDown.Y
                    );
                ptDown = e.Location;
                Invalidate(true);
            }
        }

        private void PathGradientBrushesForm_MouseUp(object sender, MouseEventArgs e)
        {
            if (bHitCenter)
            {
                bHitCenter = false;
                ptCenter = new Point(
                     ptCenter.X + e.X - ptDown.X,
                     ptCenter.Y + e.Y - ptDown.Y
                     );
                ptDown = Point.Empty;
                Invalidate(true);
            }
        }

        private bool IsNear(Point pt1, Point pt2, double delta)
        {
            double distance = Math.Sqrt(Math.Pow(pt1.X - pt2.X, 2.0)
                + Math.Pow(pt1.Y - pt2.Y, 2.0));
            return distance < delta;
        }


    }
}