using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Runtime.InteropServices;
using System.Windows.Forms;

namespace ImagesSample
{
    public partial class ColoredCursorForm : Form
    {

        [DllImport("user32.dll")]
        static extern IntPtr LoadCursorFromFile(string lpFileName);

        static Cursor ColoredCursor;

        static ColoredCursorForm()
        {
            // Load colored cursor once for the lifetime of the application
            IntPtr cursor = LoadCursorFromFile(@"c:\windows\cursors\3dgarro.cur");
            ColoredCursor = new Cursor(cursor);
        }

        public ColoredCursorForm()
        {
            InitializeComponent();
        }

        private void ColoredCursorForm_MouseEnter(object sender, EventArgs e)
        {
            // Load colored cursor
            this.Cursor = ColoredCursor;
        }

        private void ColoredCursorForm_MouseLeave(object sender, EventArgs e)
        {
            this.Cursor = Cursors.Default;
        }

        public struct IconInfo
        {
            public bool fIcon;
            public int xHotspot;
            public int yHotspot;
            public IntPtr hbmMask;
            public IntPtr hbmColor;
        }

        [DllImport("user32.dll")]
        [return: MarshalAs(UnmanagedType.Bool)]
        public static extern bool GetIconInfo(IntPtr hIcon,
            ref IconInfo pIconInfo);

        [DllImport("user32.dll")]
        public static extern IntPtr CreateIconIndirect(
            ref IconInfo icon);

        public static Cursor CreateCursor(Bitmap bmp,
              int xHotSpot, int yHotSpot)
        {
            IntPtr ptr = bmp.GetHicon();
            IconInfo tmp = new IconInfo();
            GetIconInfo(ptr, ref tmp);
            tmp.xHotspot = xHotSpot;
            tmp.yHotspot = yHotSpot;
            tmp.fIcon = false;
            ptr = CreateIconIndirect(ref tmp);
            return new Cursor(ptr);
        }

        private void buttonCreateCursor_Click(object sender, EventArgs e)
        {
            Bitmap bitmap = new Bitmap(32, 32);
            Rectangle glass = new Rectangle(10, 10, 15, 15);
            Graphics g = Graphics.FromImage(bitmap);
            using (Pen pen = new Pen(Color.Black, 2))
            {
                g.FillEllipse(Brushes.LightGray, glass);
                g.DrawEllipse(pen, glass);
                g.DrawLine(Pens.Black, 13, 17, 21, 17);
                g.DrawLine(Pens.Black, 17, 13, 17, 21);
                g.DrawLine(pen, 22, 22, 30, 30);
            }
            panelCreateCursor.Cursor = CreateCursor(bitmap, 3, 3);

            bitmap.Dispose();
        }



    }
}