using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Drawing.Drawing2D;

namespace TransformsSample
{
    public partial class WarpForm : Form
    {
        public WarpForm()
        {
            InitializeComponent();
        }

        PointF ptTopLeft;
        PointF ptTopRight;
        PointF ptBottomLeft;
        PointF ptRect;
        SizeF szRect;
        RectangleF rectPath;

        private void WarpForm_Paint(object sender, PaintEventArgs e)
        {
            PointF[] points = new PointF[] {
                AddPoint(ptTopLeft, m_ptOffsetTL),
                AddPoint(ptTopRight, m_ptOffsetTR),
                AddPoint(ptBottomLeft, m_ptOffsetBL)
            };
            using (GraphicsPath pathClip = new GraphicsPath())
            {
                pathClip.AddLines(new PointF[] {
                    points[0], points[1],
                    AddPoint(points[2], SubPoint(points[1], points[0])),
                    points[2]});
                using (Region region = new Region(pathClip))
                {
                    if (checkBoxClip.Checked)
                    {
                        e.Graphics.Clip = region;
                    }
                    RectangleF rect = new RectangleF(
                        AddPoint(ptRect, m_ptOffsetRect), AddSize(szRect, m_szOffestRect));
                    using (GraphicsPath path = new GraphicsPath())
                    using (Pen pen = new Pen(Color.Red, 2))
                    {
                        pen.StartCap = LineCap.SquareAnchor;
                        pen.EndCap = LineCap.SquareAnchor;
                        e.Graphics.DrawLine(pen, points[0], points[1]);
                        e.Graphics.DrawLine(pen, points[0], points[2]);
                        path.AddEllipse(rectPath);
                        path.Warp(points, rect);
                        e.Graphics.DrawPath(Pens.Black, path);
                    }
                }
            }
        }

        private void WarpForm_Resize(object sender, EventArgs e)
        {

        }

        float fWidth;
        float fHeight;

        public void WarpForm_Load(object sender, EventArgs e)
        {
            fWidth = ClientRectangle.Width / 3f;
            fHeight = ClientRectangle.Height / 3f;
            ptTopLeft = new PointF(fWidth, fHeight);
            ptTopRight = new PointF(fWidth * 2, fHeight);
            ptBottomLeft = new PointF(fWidth, fHeight * 2);
            ptRect = new PointF(0, 0);
            szRect = new SizeF(Width, Height);
            rectPath = new RectangleF(ptRect, szRect);
        }

        private PointF AddPoint(PointF ptDest, PointF ptSrc)
        {
            return new PointF(ptDest.X + ptSrc.X, ptDest.Y + ptSrc.Y);
        }

        private PointF SubPoint(PointF ptDest, PointF ptSrc)
        {
            return new PointF(ptDest.X - ptSrc.X, ptDest.Y - ptSrc.Y);
        }

        private SizeF AddSize(SizeF szDest, SizeF szSrc)
        {
            return new SizeF(szDest.Width + szSrc.Width, szDest.Height + szSrc.Height);
        }

        private bool AreClose(PointF pt1, PointF pt2)
        {
            return Math.Sqrt(
                Math.Pow(pt1.X - pt2.X, 2) +
                Math.Pow(pt1.Y - pt2.Y, 2)) < 0.1;

        }

        PointF m_ptOffsetTL = new PointF(0, 0);
        PointF m_ptOffsetBL = new PointF(0, 0);
        PointF m_ptOffsetTR = new PointF(0, 0);
        PointF m_ptOffsetRect = new PointF(0, 0);
        SizeF m_szOffestRect = new SizeF(0, 0);

        private void numericUpDownTopLeftX_ValueChanged(object sender, EventArgs e)
        {
            m_ptOffsetTL.X = (float)numericUpDownTopLeftX.Value;
            Invalidate(true);
        }

        private void numericUpDownTopLeftY_ValueChanged(object sender, EventArgs e)
        {
            m_ptOffsetTL.Y = (float)numericUpDownTopLeftY.Value;
            Invalidate(true);
        }

        private void numericUpDownBLX_ValueChanged(object sender, EventArgs e)
        {
            m_ptOffsetBL.X = (float)numericUpDownBLX.Value;
            Invalidate(true);

        }

        private void numericUpDownBLY_ValueChanged(object sender, EventArgs e)
        {
            m_ptOffsetBL.Y = (float)numericUpDownBLY.Value;
            Invalidate(true);
        }

        private void numericUpDownTRX_ValueChanged(object sender, EventArgs e)
        {
            m_ptOffsetTR.X = (float)numericUpDownTRX.Value;
            Invalidate(true);
        }

        private void numericUpDownTRY_ValueChanged(object sender, EventArgs e)
        {
            m_ptOffsetTR.Y = (float)numericUpDownTRY.Value;
            Invalidate(true);
        }

        private void numericUpDownRX_ValueChanged(object sender, EventArgs e)
        {
            m_ptOffsetRect.X = (float)numericUpDownRX.Value;
            Invalidate(true);
        }

        private void numericUpDownRY_ValueChanged(object sender, EventArgs e)
        {
            m_ptOffsetRect.Y = (float)numericUpDownRY.Value;
            Invalidate(true);
        }

        private void numericUpDownRW_ValueChanged(object sender, EventArgs e)
        {
            m_szOffestRect.Width = (float)numericUpDownRW.Value;
            Invalidate(true);
        }

        private void numericUpDownRH_ValueChanged(object sender, EventArgs e)
        {
            m_szOffestRect.Height = (float)numericUpDownRH.Value;
            Invalidate(true);
        }

        private void checkBoxClip_CheckedChanged(object sender, EventArgs e)
        {
            Invalidate(true);
        }
    }
}