using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Printing;
using System.Text;
using System.Windows.Forms;
using System.Drawing.Drawing2D;
using Utils4226;

namespace Ch8
{
    public partial class MainForm : Form
    {
        String fileName = "myFile.txt"; 
        bool m_bPreview = false;
        int m_nPage = 1;
        int m_nTotalPage = 4;
        int m_nMaxPage = 4;
        Font m_fontPrinter = null;
        Pen m_penBlack = null;
        Color[] colors = new Color[]{ Color.Red, Color.Blue, Color.Green, Color.Black};

        public MainForm()
        {
            InitializeComponent();

            m_nPage = 1;
            printPreviewControl1.StartPage = 0;
            printPreviewControl1.Rows = 1;
            printPreviewControl1.Columns = 2;
        }

        private void buttonPrint_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show("Do you want to print four pages of nothing?",
                    "Are you sure?",
                    MessageBoxButtons.YesNo) == DialogResult.Yes)
            {
                m_nPage = 1;
                this.printDocument.DocumentName = this.fileName;
                this.printDocument.Print();
            }
        }

        private void PrintHelper(Graphics g, System.Drawing.Printing.PrintPageEventArgs e)
        {
            RectangleF realPageBounds = Printing.GetRealPageBounds(e, m_bPreview);
            RectangleF realMarginBounds = Printing.GetRealMarginBounds(e, m_bPreview);
            
            g.DrawRectangle(m_penBlack,
                realPageBounds.X, realPageBounds.Y,
                realPageBounds.Width, realPageBounds.Height);
            g.DrawRectangle(m_penBlack,
                realMarginBounds.X, realMarginBounds.Y,
                realMarginBounds.Width, realMarginBounds.Height);            
            
            g.DrawString("COP4226", m_fontPrinter, Brushes.Black, realPageBounds);

            g.DrawString("Hello, Printer", m_fontPrinter,
                Brushes.Black, realMarginBounds);
            realMarginBounds.Y += m_fontPrinter.GetHeight(g);
            g.DrawString("Page: " + m_nPage.ToString(), m_fontPrinter,
                Brushes.Black, realMarginBounds);

            using (StringFormat format = new StringFormat())
            {
                format.Alignment = StringAlignment.Far;
                format.LineAlignment = StringAlignment.Far;
                g.DrawString("FIU", m_fontPrinter, Brushes.Black, realPageBounds, format);
            }
            m_nPage++;
            e.HasMorePages = m_nPage <= m_nMaxPage;
        }

        private void printDocument_PrintPage(object sender, System.Drawing.Printing.PrintPageEventArgs e)
        {
            e.PageSettings.Landscape = false;
            if (checkBoxInches.Checked) {
                e.Graphics.PageUnit = GraphicsUnit.Inch;
                e.Graphics.PageScale = 1;               
            }
            if (checkBoxTwoUp.Checked)
            {
                RectangleF realPageBounds = Printing.GetRealPageBounds(e, m_bPreview);
                Matrix mat = new Matrix();
                mat.Scale(0.5f, 0.5f);
                e.Graphics.Transform = mat;
                PrintHelper(e.Graphics, e);
                if (!e.HasMorePages) return;
                mat.Translate(realPageBounds.Width, 0);
                e.Graphics.Transform = mat;
                PrintHelper(e.Graphics, e);
                if (!e.HasMorePages) return;
                mat.Translate(
                    -realPageBounds.Width,
                    realPageBounds.Height);
                e.Graphics.Transform = mat;
                PrintHelper(e.Graphics, e);
                if (!e.HasMorePages) return;
                mat.Translate(
                    realPageBounds.Width, 
                    0);
                e.Graphics.Transform = mat;
            }
            PrintHelper(e.Graphics, e);
        }

        private void printDocument_BeginPrint(object sender, System.Drawing.Printing.PrintEventArgs e)
        {
            m_bPreview = (e.PrintAction == PrintAction.PrintToPreview);
            m_fontPrinter = new Font("Lucida Console", 36);
            float fPenWidth = (float)numericUpDownPenW.Value;
            if (checkBoxInches.Checked)
            {
                fPenWidth = Conversion.PixelsToInches(this, fPenWidth);
            }
            m_penBlack = new Pen(Color.Red, fPenWidth);
            if (checkBoxInset.Checked)
            {
                m_penBlack.Alignment = PenAlignment.Inset;
            }           
        }

        private void printDocument_EndPrint(object sender, System.Drawing.Printing.PrintEventArgs e)
        {
            m_fontPrinter.Dispose();
            m_fontPrinter = null;
            m_penBlack.Dispose();
            m_penBlack = null;
            m_nPage = 1;
        }

        private void buttonPreview_Click(object sender, EventArgs e)
        {
            m_nPage = 1;
            printPreviewDialog.ShowDialog();
        }

        private void buttonNext_Click(object sender, EventArgs e)
        {
            if (m_nPage < m_nTotalPage)
            {
                m_nPage++;
            }
            printPreviewControl1.StartPage = m_nPage - 1;
        }

        private void printDocument_QueryPageSettings(object sender, QueryPageSettingsEventArgs e)
        {
            //If changed here, then margins in pageSetup will be ignored.         
            //e.PageSettings.Margins = new Margins(m_nMargin, m_nMargin, m_nMargin, m_nMargin);
        }

        int m_nMargin = 100;
        private void buttonMargin_Click(object sender, EventArgs e)
        {
            m_nMargin = 50;
            pageSetupDialog.PageSettings.Margins = new Margins(m_nMargin, m_nMargin, m_nMargin, m_nMargin);
            printPreviewControl1.InvalidatePreview();
        }

        private void helpButtonReset_Click(object sender, EventArgs e)
        {
            m_nMargin = 100;
            pageSetupDialog.PageSettings.Margins = new Margins(m_nMargin, m_nMargin, m_nMargin, m_nMargin);
            printPreviewControl1.InvalidatePreview();
        }

        private void buttonSetup_Click(object sender, EventArgs e)
        {
            pageSetupDialog.ShowDialog();
        }

        private void buttonDialog_Click(object sender, EventArgs e)
        {
            printDocument.PrinterSettings.FromPage = 1;
            printDocument.PrinterSettings.ToPage = m_nTotalPage;
            printDocument.PrinterSettings.MinimumPage = 1;
            printDocument.PrinterSettings.MaximumPage = m_nTotalPage;
            printDialog.AllowSomePages = true;
            if (printDialog.ShowDialog() == DialogResult.OK)
            {
                if (printDialog.PrinterSettings.PrintRange ==
                    PrintRange.SomePages)
                {
                    m_nPage = printDocument.PrinterSettings.FromPage;
                    m_nMaxPage = printDocument.PrinterSettings.ToPage;
                }
                else
                {
                    m_nPage = 1;
                    m_nMaxPage = m_nTotalPage;
                }
                this.printDocument.Print();
            }
        }

        private void helpButtonRedraw_Click(object sender, EventArgs e)
        {
            printPreviewControl1.InvalidatePreview();
        }

    }
}